@@ -2,7 +2,7 @@
 # RMM, 10 Nov 2012
 
 from .exception import ControlNotImplemented, ControlSlycot
-from .namedio import issiso
+from .iosys import issiso
 from .statesp import StateSpace, _convert_to_statespace
 from .statefbk import ctrb, obsv
 
@@ -19,7 +19,7 @@ __all__ = ['canonical_form', 'reachable_form', 'observable_form', 'modal_form',
 
 
 def canonical_form(xsys, form='reachable'):
-    """Convert a system into canonical form
+    """Convert a system into canonical form.
 
     Parameters
     ----------
@@ -71,7 +71,7 @@ def canonical_form(xsys, form='reachable'):
 
 # Reachable canonical form
 def reachable_form(xsys):
-    """Convert a system into reachable canonical form
+    """Convert a system into reachable canonical form.
 
     Parameters
     ----------
@@ -134,7 +134,7 @@ def reachable_form(xsys):
 
 
 def observable_form(xsys):
-    """Convert a system into observable canonical form
+    """Convert a system into observable canonical form.
 
     Parameters
     ----------
@@ -204,7 +204,7 @@ def similarity_transform(xsys, T, timescale=1, inverse=False):
         The matrix `T` defines the new set of coordinates z = T x.
     timescale : float, optional
         If present, also rescale the time unit to tau = timescale * t
-    inverse: boolean, optional
+    inverse : bool, optional
         If True (default), transform so z = T x.  If False, transform
         so x = T z.
 
@@ -255,7 +255,7 @@ _PMAX_SEARCH_TOL = 1.001
 
 
 def _bdschur_defective(blksizes, eigvals):
-    """Check  for defective modal decomposition
+    """Check  for defective modal decomposition.
 
     Parameters
     ----------
@@ -290,7 +290,7 @@ def _bdschur_defective(blksizes, eigvals):
 
 
 def _bdschur_condmax_search(aschur, tschur, condmax):
-    """Block-diagonal Schur decomposition search up to condmax
+    """Block-diagonal Schur decomposition search up to condmax.
 
     Iterates mb03rd with different pmax values until:
       - result is non-defective;
@@ -393,25 +393,25 @@ def _bdschur_condmax_search(aschur, tschur, condmax):
 
 
 def bdschur(a, condmax=None, sort=None):
-    """Block-diagonal Schur decomposition
+    """Block-diagonal Schur decomposition.
 
     Parameters
     ----------
-        a : (M, M) array_like
-            Real matrix to decompose
-        condmax : None or float, optional
-            If None (default), use 1/sqrt(eps), which is approximately 1e8
-        sort : {None, 'continuous', 'discrete'}
-            Block sorting; see below.
+    a : (M, M) array_like
+        Real matrix to decompose
+    condmax : None or float, optional
+        If None (default), use 1/sqrt(eps), which is approximately 1e8
+    sort : {None, 'continuous', 'discrete'}
+        Block sorting; see below.
 
     Returns
     -------
-        amodal : (M, M) real ndarray
-            Block-diagonal Schur decomposition of `a`
-        tmodal : (M, M) real ndarray
-            Similarity transform relating `a` and `amodal`
-        blksizes : (N,) int ndarray
-            Array of Schur block sizes
+    amodal : (M, M) real ndarray
+        Block-diagonal Schur decomposition of `a`
+    tmodal : (M, M) real ndarray
+        Similarity transform relating `a` and `amodal`
+    blksizes : (N,) int ndarray
+        Array of Schur block sizes
 
     Notes
     -----
@@ -482,7 +482,7 @@ def bdschur(a, condmax=None, sort=None):
 
 
 def modal_form(xsys, condmax=None, sort=False):
-    """Convert a system into modal canonical form
+    """Convert a system into modal canonical form.
 
     Parameters
     ----------
