@@ -1,41 +1,8 @@
-# Copyright (c) 2010 by California Institute of Technology
-# Copyright (c) 2012 by Delft University of Technology
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-#
-# 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-#
-# 2. Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in the
-#    documentation and/or other materials provided with the distribution.
-#
-# 3. Neither the names of the California Institute of Technology nor
-#    the Delft University of Technology nor
-#    the names of its contributors may be used to endorse or promote
-#    products derived from this software without specific prior
-#    written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
-# FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL CALTECH
-# OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
-# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-# SUCH DAMAGE.
+# frdata.py - frequency response data representation and functions
 #
 # Author: M.M. (Rene) van Paassen (using xferfcn.py as basis)
 # Date: 02 Oct 12
 
-
 """
 Frequency response data representation and functions.
 
@@ -43,19 +10,18 @@ This module contains the FRD class and also functions that operate on
 FRD data.
 """
 
-# External function declarations
 from copy import copy
 from warnings import warn
 
 import numpy as np
-from numpy import angle, array, empty, ones, \
-    real, imag, absolute, eye, linalg, where, sort
-from scipy.interpolate import splprep, splev
+from numpy import absolute, angle, array, empty, eye, imag, linalg, ones, \
+    real, sort, where
+from scipy.interpolate import splev, splprep
 
-from .lti import LTI, _process_frequency_response
-from .exception import pandas_check
-from .namedio import NamedIOSystem, _process_namedio_keywords
 from . import config
+from .exception import pandas_check
+from .iosys import InputOutputSystem, _process_iosys_keywords, common_timebase
+from .lti import LTI, _process_frequency_response
 
 __all__ = ['FrequencyResponseData', 'FRD', 'frd']
 
@@ -66,7 +32,9 @@ class FrequencyResponseData(LTI):
     A class for models defined by frequency response data (FRD).
 
     The FrequencyResponseData (FRD) class is used to represent systems in
-    frequency response data form.
+    frequency response data form.  It can be created manually using the
+    class constructor, using the :func:~~control.frd` factory function
+    (preferred), or via the :func:`~control.frequency_response` function.
 
     Parameters
     ----------
@@ -78,6 +46,8 @@ class FrequencyResponseData(LTI):
         corresponding to the frequency points in omega
     w : iterable of real frequencies
         List of frequency points for which data are available.
+    sysname : str or None
+        Name of the system that generated the data.
     smooth : bool, optional
         If ``True``, create an interpolation function that allows the
         frequency response to be computed at any frequency within the range of
@@ -93,6 +63,12 @@ class FrequencyResponseData(LTI):
     fresp : 3D array
         Frequency response, indexed by output index, input index, and
         frequency point.
+    dt : float, True, or None
+        System timebase.
+
+    See Also
+    --------
+    frd
 
     Notes
     -----
@@ -114,11 +90,6 @@ class FrequencyResponseData(LTI):
     for a more detailed description.
 
     """
-
-    # Allow NDarray * StateSpace to give StateSpace._rmul_() priority
-    # https://docs.scipy.org/doc/numpy/reference/arrays.classes.html
-    __array_priority__ = 13     # override ndarray, StateSpace, I/O sys
-
     #
     # Class attributes
     #
@@ -173,6 +144,7 @@ class FrequencyResponseData(LTI):
                 else:
                     z = np.exp(1j * self.omega * otherlti.dt)
                     self.fresp = otherlti(z, squeeze=False)
+                arg_dt = otherlti.dt
 
             else:
                 # The user provided a response and a freq vector
@@ -186,6 +158,7 @@ class FrequencyResponseData(LTI):
                         "The frequency data constructor needs a 1-d or 3-d"
                         " response data array and a matching frequency vector"
                         " size")
+                arg_dt = None
 
         elif len(args) == 1:
             # Use the copy constructor.
@@ -195,31 +168,46 @@ class FrequencyResponseData(LTI):
                     " an FRD object.  Received %s." % type(args[0]))
             self.omega = args[0].omega
             self.fresp = args[0].fresp
+            arg_dt = args[0].dt
+
         else:
             raise ValueError(
                 "Needs 1 or 2 arguments; received %i." % len(args))
 
         #
-        # Process key word arguments
+        # Process keyword arguments
         #
+
+        # If data was generated by a system, keep track of that (used when
+        # plotting data).  Otherwise, use the system name, if given.
+        self.sysname = kwargs.pop('sysname', kwargs.get('name', None))
+
+        # Keep track of default properties for plotting
+        self.plot_phase = kwargs.pop('plot_phase', None)
+        self.title = kwargs.pop('title', None)
+        self.plot_type = kwargs.pop('plot_type', 'bode')
+
         # Keep track of return type
         self.return_magphase=kwargs.pop('return_magphase', False)
         if self.return_magphase not in (True, False):
             raise ValueError("unknown return_magphase value")
+        self._return_singvals=kwargs.pop('_return_singvals', False)
 
         # Determine whether to squeeze the output
         self.squeeze=kwargs.pop('squeeze', None)
         if self.squeeze not in (None, True, False):
             raise ValueError("unknown squeeze value")
 
-        # Process namedio keywords
+        # Process iosys keywords
         defaults = {
-            'inputs': self.fresp.shape[1], 'outputs': self.fresp.shape[0]}
-        name, inputs, outputs, states, dt = _process_namedio_keywords(
+            'inputs': self.fresp.shape[1], 'outputs': self.fresp.shape[0],
+            'dt': None}
+        name, inputs, outputs, states, dt = _process_iosys_keywords(
                 kwargs, defaults, end=True)
+        dt = common_timebase(dt, arg_dt)        # choose compatible timebase
 
         # Process signal names
-        NamedIOSystem.__init__(
+        InputOutputSystem.__init__(
             self, name=name, inputs=inputs, outputs=outputs, dt=dt)
 
         # create interpolation functions
@@ -262,7 +250,7 @@ class FrequencyResponseData(LTI):
         """String representation of the transfer function."""
 
         mimo = self.ninputs > 1 or self.noutputs > 1
-        outstr = ['Frequency response data']
+        outstr = [f"{InputOutputSystem.__str__(self)}"]
 
         for i in range(self.ninputs):
             for j in range(self.noutputs):
@@ -304,7 +292,7 @@ class FrequencyResponseData(LTI):
 
         # Convert the second argument to a frequency response function.
         # or re-base the frd to the current omega (if needed)
-        other = _convert_to_FRD(other, omega=self.omega)
+        other = _convert_to_frd(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
         if self.ninputs != other.ninputs:
@@ -341,7 +329,7 @@ class FrequencyResponseData(LTI):
             return FRD(self.fresp * other, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convert_to_FRD(other, omega=self.omega)
+            other = _convert_to_frd(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
         if self.ninputs != other.noutputs:
@@ -368,7 +356,7 @@ class FrequencyResponseData(LTI):
             return FRD(self.fresp * other, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convert_to_FRD(other, omega=self.omega)
+            other = _convert_to_frd(other, omega=self.omega)
 
         # Check that the input-output sizes are consistent.
         if self.noutputs != other.ninputs:
@@ -396,7 +384,7 @@ class FrequencyResponseData(LTI):
             return FRD(self.fresp * (1/other), self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convert_to_FRD(other, omega=self.omega)
+            other = _convert_to_frd(other, omega=self.omega)
 
         if (self.ninputs > 1 or self.noutputs > 1 or
             other.ninputs > 1 or other.noutputs > 1):
@@ -415,7 +403,7 @@ class FrequencyResponseData(LTI):
             return FRD(other / self.fresp, self.omega,
                        smooth=(self.ifunc is not None))
         else:
-            other = _convert_to_FRD(other, omega=self.omega)
+            other = _convert_to_frd(other, omega=self.omega)
 
         if (self.ninputs > 1 or self.noutputs > 1 or
             other.ninputs > 1 or other.noutputs > 1):
@@ -554,8 +542,8 @@ class FrequencyResponseData(LTI):
         ------
         ValueError
             If `s` is not purely imaginary, because
-            :class:`FrequencyDomainData` systems are only defined at imaginary
-            frequency values.
+            :class:`FrequencyResponseData` systems are only defined at
+            imaginary values (corresponding to real frequencies).
 
         """
         if s is None:
@@ -587,7 +575,10 @@ class FrequencyResponseData(LTI):
     def __iter__(self):
         fresp = _process_frequency_response(
             self, self.omega, self.fresp, squeeze=self.squeeze)
-        if not self.return_magphase:
+        if self._return_singvals:
+            # Legacy processing for singular values
+            return iter((self.fresp[:, 0, :], self.omega))
+        elif not self.return_magphase:
             return iter((self.omega, fresp))
         return iter((np.abs(fresp), np.angle(fresp), self.omega))
 
@@ -606,18 +597,19 @@ class FrequencyResponseData(LTI):
             Method has been given the more pythonic name
             :meth:`FrequencyResponseData.frequency_response`. Or use
             :func:`freqresp` in the MATLAB compatibility module.
+
         """
         warn("FrequencyResponseData.freqresp(omega) will be removed in a "
              "future release of python-control; use "
              "FrequencyResponseData.frequency_response(omega), or "
              "freqresp(sys, omega) in the MATLAB compatibility module "
-             "instead", DeprecationWarning)
+             "instead", FutureWarning)
         return self.frequency_response(omega)
 
     def feedback(self, other=1, sign=-1):
         """Feedback interconnection between two FRD objects."""
 
-        other = _convert_to_FRD(other, omega=self.omega)
+        other = _convert_to_frd(other, omega=self.omega)
 
         if (self.noutputs != other.ninputs or self.ninputs != other.noutputs):
             raise ValueError(
@@ -634,8 +626,42 @@ class FrequencyResponseData(LTI):
 
         return FRD(fresp, other.omega, smooth=(self.ifunc is not None))
 
+    # Plotting interface
+    def plot(self, plot_type=None, *args, **kwargs):
+        """Plot the frequency response using a Bode plot.
+
+        Plot the frequency response using either a standard Bode plot
+        (default) or using a singular values plot (by setting `plot_type`
+        to 'svplot').  See :func:`~control.bode_plot` and
+        :func:`~control.singular_values_plot` for more detailed
+        descriptions.
+
+        """
+        from .freqplot import bode_plot, singular_values_plot
+        from .nichols import nichols_plot
+
+        if plot_type is None:
+            plot_type = self.plot_type
+
+        if plot_type == 'bode':
+            return bode_plot(self, *args, **kwargs)
+        elif plot_type == 'nichols':
+            return nichols_plot(self, *args, **kwargs)
+        elif plot_type == 'svplot':
+            return singular_values_plot(self, *args, **kwargs)
+        else:
+            raise ValueError(f"unknown plot type '{plot_type}'")
+
     # Convert to pandas
     def to_pandas(self):
+        """Convert response data to pandas data frame.
+
+        Creates a pandas data frame for the value of the frequency
+        response at each `omega`.  The frequency response values are
+        labeled in the form "H_{<out>, <in>}" where "<out>" and "<in>"
+        are replaced with the output and input labels for the system.
+
+        """
         if not pandas_check():
             ImportError('pandas not installed')
         import pandas
@@ -663,7 +689,7 @@ class FrequencyResponseData(LTI):
 FRD = FrequencyResponseData
 
 
-def _convert_to_FRD(sys, omega, inputs=1, outputs=1):
+def _convert_to_frd(sys, omega, inputs=1, outputs=1):
     """Convert a system to frequency response data form (if needed).
 
     If sys is already an frd, and its frequency range matches or
@@ -674,14 +700,14 @@ def _convert_to_FRD(sys, omega, inputs=1, outputs=1):
     manually, as in:
 
     >>> import numpy as np
-    >>> from control.frdata import _convert_to_FRD
+    >>> from control.frdata import _convert_to_frd
 
     >>> omega = np.logspace(-1, 1)
-    >>> frd = _convert_to_FRD(3., omega) # Assumes inputs = outputs = 1
+    >>> frd = _convert_to_frd(3., omega) # Assumes inputs = outputs = 1
     >>> frd.ninputs, frd.noutputs
     (1, 1)
 
-    >>> frd = _convert_to_FRD(1., omega, inputs=3, outputs=2)
+    >>> frd = _convert_to_frd(1., omega, inputs=3, outputs=2)
     >>> frd.ninputs, frd.noutputs
     (3, 2)
 
@@ -730,51 +756,67 @@ def _convert_to_FRD(sys, omega, inputs=1, outputs=1):
                     sys.__class__)
 
 
-def frd(*args):
-    """frd(d, w)
+def frd(*args, **kwargs):
+    """frd(response, omega[, dt])
 
-    Construct a frequency response data model
+    Construct a frequency response data (FRD) model.
 
-    frd models store the (measured) frequency response of a system.
+    A frequency response data model stores the (measured) frequency response
+    of a system.  This factory function can be called in different ways:
 
-    This function can be called in different ways:
-
-    ``frd(response, freqs)``
+    ``frd(response, omega)``
         Create an frd model with the given response data, in the form of
-        complex response vector, at matching frequency freqs [in rad/s]
+        complex response vector, at matching frequencies ``omega`` [in rad/s].
 
-    ``frd(sys, freqs)``
+    ``frd(sys, omega)``
         Convert an LTI system into an frd model with data at frequencies
-        freqs.
+        ``omega``.
 
     Parameters
     ----------
-    response: array_like, or list
-        complex vector with the system response
-    freq: array_lik or lis
-        vector with frequencies
-    sys: LTI (StateSpace or TransferFunction)
-        A linear system
+    response : array_like or LTI system
+        Complex vector with the system response or an LTI system that can
+        be used to copmute the frequency response at a list of frequencies.
+    omega : array_like
+        Vector of frequencies at which the response is evaluated.
+    dt : float, True, or None
+        System timebase.
+    smooth : bool, optional
+        If ``True``, create an interpolation function that allows the
+        frequency response to be computed at any frequency within the range
+        of frequencies give in ``omega``.  If ``False`` (default),
+        frequency response can only be obtained at the frequencies
+        specified in ``omega``.
 
     Returns
     -------
-    sys: FRD
-        New frequency response system
+    sys : :class:`FrequencyResponseData`
+        New frequency response data system.
+
+    Other Parameters
+    ----------------
+    inputs, outputs : str, or list of str, optional
+        List of strings that name the individual signals of the transformed
+        system.  If not given, the inputs and outputs are the same as the
+        original system.
+    name : string, optional
+        System name. If unspecified, a generic name <sys[id]> is generated
+        with a unique integer id.
 
     See Also
     --------
-    FRD, ss, tf
+    FrequencyResponseData, frequency_response, ss, tf
 
     Examples
     --------
     >>> # Create from measurements
     >>> response = [1.0, 1.0, 0.5]
-    >>> freqs = [1, 10, 100]
-    >>> F = ct.frd(response, freqs)
+    >>> omega = [1, 10, 100]
+    >>> F = ct.frd(response, omega)
 
     >>> G = ct.tf([1], [1, 1])
-    >>> freqs = [1, 10, 100]
-    >>> F = ct.frd(G, freqs)
+    >>> omega = [1, 10, 100]
+    >>> F = ct.frd(G, omega)
 
     """
-    return FRD(*args)
+    return FrequencyResponseData(*args, **kwargs)
