@@ -5,97 +5,36 @@ and TransferFunction.  It is designed for use in the python-control library.
 """
 
 import numpy as np
+import math
 
 from numpy import real, angle, abs
 from warnings import warn
 from . import config
-from .namedio import NamedIOSystem
+from .iosys import InputOutputSystem
 
 __all__ = ['poles', 'zeros', 'damp', 'evalfr', 'frequency_response',
-           'freqresp', 'dcgain', 'bandwidth', 'pole', 'zero']
+           'freqresp', 'dcgain', 'bandwidth', 'LTI']
 
 
-class LTI(NamedIOSystem):
+class LTI(InputOutputSystem):
     """LTI is a parent class to linear time-invariant (LTI) system objects.
 
     LTI is the parent to the StateSpace and TransferFunction child classes. It
     contains the number of inputs and outputs, and the timebase (dt) for the
     system.  This function is not generally called directly by the user.
 
-    The timebase for the system, dt, is used to specify whether the system
-    is operating in continuous or discrete time. It can have the following
-    values:
-
-      * dt = None       No timebase specified
-      * dt = 0          Continuous time system
-      * dt > 0          Discrete time system with sampling time dt
-      * dt = True       Discrete time system with unspecified sampling time
-
     When two LTI systems are combined, their timebases much match.  A system
     with timebase None can be combined with a system having a specified
     timebase, and the result will have the timebase of the latter system.
 
-    Note: dt processing has been moved to the NamedIOSystem class.
+    Note: dt processing has been moved to the InputOutputSystem class.
 
     """
-
     def __init__(self, inputs=1, outputs=1, states=None, name=None, **kwargs):
         """Assign the LTI object's numbers of inputs and ouputs."""
         super().__init__(
             name=name, inputs=inputs, outputs=outputs, states=states, **kwargs)
 
-    #
-    # Getter and setter functions for legacy state attributes
-    #
-    # For this iteration, generate a deprecation warning whenever the
-    # getter/setter is called.  For a future iteration, turn it into a
-    # future warning, so that users will see it.
-    #
-
-    def _get_inputs(self):
-        warn("The LTI `inputs` attribute will be deprecated in a future "
-             "release.  Use `ninputs` instead.",
-             DeprecationWarning, stacklevel=2)
-        return self.ninputs
-
-    def _set_inputs(self, value):
-        warn("The LTI `inputs` attribute will be deprecated in a future "
-             "release.  Use `ninputs` instead.",
-             DeprecationWarning, stacklevel=2)
-        self.ninputs = value
-
-    #: Deprecated
-    inputs = property(
-        _get_inputs, _set_inputs, doc="""
-        Deprecated attribute; use :attr:`ninputs` instead.
-
-        The ``inputs`` attribute was used to store the number of system
-        inputs.  It is no longer used.  If you need access to the number
-        of inputs for an LTI system, use :attr:`ninputs`.
-        """)
-
-    def _get_outputs(self):
-        warn("The LTI `outputs` attribute will be deprecated in a future "
-             "release.  Use `noutputs` instead.",
-             DeprecationWarning, stacklevel=2)
-        return self.noutputs
-
-    def _set_outputs(self, value):
-        warn("The LTI `outputs` attribute will be deprecated in a future "
-             "release.  Use `noutputs` instead.",
-             DeprecationWarning, stacklevel=2)
-        self.noutputs = value
-
-    #: Deprecated
-    outputs = property(
-        _get_outputs, _set_outputs, doc="""
-        Deprecated attribute; use :attr:`noutputs` instead.
-
-        The ``outputs`` attribute was used to store the number of system
-        outputs.  It is no longer used.  If you need access to the number of
-        outputs for an LTI system, use :attr:`noutputs`.
-        """)
-
     def damp(self):
         '''Natural frequency, damping ratio of system poles
 
@@ -118,16 +57,16 @@ class LTI(NamedIOSystem):
         zeta = -real(splane_poles)/wn
         return wn, zeta, poles
 
-    def frequency_response(self, omega, squeeze=None):
+    def frequency_response(self, omega=None, squeeze=None):
         """Evaluate the linear time-invariant system at an array of angular
         frequencies.
 
-        Reports the frequency response of the system,
+        For continuous time systems, computes the frequency response as
 
              G(j*omega) = mag * exp(j*phase)
 
-        for continuous time systems. For discrete time systems, the response
-        is evaluated around the unit circle such that
+        For discrete time systems, the response is evaluated around the
+        unit circle such that
 
              G(exp(j*omega*dt)) = mag * exp(j*phase).
 
@@ -149,23 +88,25 @@ class LTI(NamedIOSystem):
 
         Returns
         -------
-        response : :class:`FrequencyReponseData`
+        response : :class:`FrequencyResponseData`
             Frequency response data object representing the frequency
             response.  This object can be assigned to a tuple using
 
                 mag, phase, omega = response
 
-            where ``mag`` is the magnitude (absolute value, not dB or
-            log10) of the system frequency response, ``phase`` is the wrapped
-            phase in radians of the system frequency response, and ``omega``
-            is the (sorted) frequencies at which the response was evaluated.
+            where ``mag`` is the magnitude (absolute value, not dB or log10)
+            of the system frequency response, ``phase`` is the wrapped phase
+            in radians of the system frequency response, and ``omega`` is
+            the (sorted) frequencies at which the response was evaluated.
             If the system is SISO and squeeze is not True, ``magnitude`` and
-            ``phase`` are 1D, indexed by frequency.  If the system is not SISO
-            or squeeze is False, the array is 3D, indexed by the output,
-            input, and frequency.  If ``squeeze`` is True then
-            single-dimensional axes are removed.
+            ``phase`` are 1D, indexed by frequency.  If the system is not
+            SISO or squeeze is False, the array is 3D, indexed by the
+            output, input, and, if omega is array_like, frequency.  If
+            ``squeeze`` is True then single-dimensional axes are removed.
 
         """
+        from .frdata import FrequencyResponseData
+
         omega = np.sort(np.array(omega, ndmin=1))
         if self.isdtime(strict=True):
             # Convert the frequency to discrete time
@@ -176,10 +117,11 @@ class LTI(NamedIOSystem):
             s = 1j * omega
 
         # Return the data as a frequency response data object
-        from .frdata import FrequencyResponseData
-        response = self.__call__(s)
+        response = self(s)
         return FrequencyResponseData(
-            response, omega, return_magphase=True, squeeze=squeeze)
+            response, omega, return_magphase=True, squeeze=squeeze,
+            dt=self.dt, sysname=self.name, inputs=self.input_labels,
+            outputs=self.output_labels, plot_type='bode')
 
     def dcgain(self):
         """Return the zero-frequency gain"""
@@ -261,20 +203,6 @@ class LTI(NamedIOSystem):
         from control.passivity import ispassive
         return ispassive(self)
 
-    #
-    # Deprecated functions
-    #
-
-    def pole(self):
-        warn("pole() will be deprecated; use poles()",
-             PendingDeprecationWarning)
-        return self.poles()
-
-    def zero(self):
-        warn("zero() will be deprecated; use zeros()",
-             PendingDeprecationWarning)
-        return self.zeros()
-
 
 def poles(sys):
     """
@@ -282,12 +210,12 @@ def poles(sys):
 
     Parameters
     ----------
-    sys: StateSpace or TransferFunction
+    sys : StateSpace or TransferFunction
         Linear system
 
     Returns
     -------
-    poles: ndarray
+    poles : ndarray
         Array that contains the system's poles.
 
     See Also
@@ -301,18 +229,13 @@ def poles(sys):
     return sys.poles()
 
 
-def pole(sys):
-    warn("pole() will be deprecated; use poles()", PendingDeprecationWarning)
-    return poles(sys)
-
-
 def zeros(sys):
     """
     Compute system zeros.
 
     Parameters
     ----------
-    sys: StateSpace or TransferFunction
+    sys : StateSpace or TransferFunction
         Linear system
 
     Returns
@@ -331,14 +254,9 @@ def zeros(sys):
     return sys.zeros()
 
 
-def zero(sys):
-    warn("zero() will be deprecated; use zeros()", PendingDeprecationWarning)
-    return zeros(sys)
-
-
 def damp(sys, doprint=True):
     """
-    Compute natural frequencies, damping ratios, and poles of a system
+    Compute natural frequencies, damping ratios, and poles of a system.
 
     Parameters
     ----------
@@ -412,7 +330,7 @@ def evalfr(sys, x, squeeze=None):
 
     Parameters
     ----------
-    sys: StateSpace or TransferFunction
+    sys : StateSpace or TransferFunction
         Linear system
     x : complex scalar or 1D array_like
         Complex frequency(s)
@@ -451,10 +369,12 @@ def evalfr(sys, x, squeeze=None):
     .. todo:: Add example with MIMO system
 
     """
-    return sys.__call__(x, squeeze=squeeze)
+    return sys(x, squeeze=squeeze)
 
 
-def frequency_response(sys, omega, squeeze=None):
+def frequency_response(
+        sysdata, omega=None, omega_limits=None, omega_num=None,
+        Hz=None, squeeze=None):
     """Frequency response of an LTI system at multiple angular frequencies.
 
     In general the system may be multiple input, multiple output (MIMO), where
@@ -463,22 +383,25 @@ def frequency_response(sys, omega, squeeze=None):
 
     Parameters
     ----------
-    sys: StateSpace or TransferFunction
-        Linear system
-    omega : float or 1D array_like
-        A list of frequencies in radians/sec at which the system should be
-        evaluated. The list can be either a python list or a numpy array
-        and will be sorted before evaluation.
-    squeeze : bool, optional
-        If squeeze=True, remove single-dimensional entries from the shape of
-        the output even if the system is not SISO. If squeeze=False, keep all
-        indices (output, input and, if omega is array_like, frequency) even if
-        the system is SISO. The default value can be set using
-        config.defaults['control.squeeze_frequency_response'].
+    sysdata : LTI system or list of LTI systems
+        Linear system(s) for which frequency response is computed.
+    omega : float or 1D array_like, optional
+        Frequencies in radians/sec at which the system should be
+        evaluated. Can be a single frequency or array of frequencies, which
+        will be sorted before evaluation.  If None (default), a common set
+        of frequencies that works across all given systems is computed.
+    omega_limits : array_like of two values, optional
+        Limits to the range of frequencies, in rad/sec. Specifying
+        ``omega`` as a list of two elements is equivalent to providing
+        ``omega_limits``.  Ignored if omega is provided.
+    omega_num : int, optional
+        Number of frequency samples at which to compute the response.
+        Defaults to config.defaults['freqplot.number_of_samples'].  Ignored
+        if omega is provided.
 
     Returns
     -------
-    response : FrequencyResponseData
+    response : :class:`FrequencyResponseData`
         Frequency response data object representing the frequency response.
         This object can be assigned to a tuple using
 
@@ -488,21 +411,49 @@ def frequency_response(sys, omega, squeeze=None):
         the system frequency response, ``phase`` is the wrapped phase in
         radians of the system frequency response, and ``omega`` is the
         (sorted) frequencies at which the response was evaluated.  If the
-        system is SISO and squeeze is not True, ``magnitude`` and ``phase``
+        system is SISO and squeeze is not False, ``magnitude`` and ``phase``
         are 1D, indexed by frequency.  If the system is not SISO or squeeze
         is False, the array is 3D, indexed by the output, input, and
         frequency.  If ``squeeze`` is True then single-dimensional axes are
         removed.
 
+        Returns a list of :class:`FrequencyResponseData` objects if sys is
+        a list of systems.
+
+    Other Parameters
+    ----------------
+    Hz : bool, optional
+        If True, when computing frequency limits automatically set
+        limits to full decades in Hz instead of rad/s. Omega is always
+        returned in rad/sec.
+    squeeze : bool, optional
+        If squeeze=True, remove single-dimensional entries from the shape of
+        the output even if the system is not SISO. If squeeze=False, keep all
+        indices (output, input and, if omega is array_like, frequency) even if
+        the system is SISO. The default value can be set using
+        config.defaults['control.squeeze_frequency_response'].
+
     See Also
     --------
     evalfr
-    bode
+    bode_plot
 
     Notes
     -----
-    This function is a wrapper for :meth:`StateSpace.frequency_response` and
-    :meth:`TransferFunction.frequency_response`.
+    1. This function is a wrapper for :meth:`StateSpace.frequency_response`
+       and :meth:`TransferFunction.frequency_response`.
+
+    2. You can also use the lower-level methods ``sys(s)`` or ``sys(z)`` to
+       generate the frequency response for a single system.
+
+    3. All frequency data should be given in rad/sec.  If frequency limits
+       are computed automatically, the `Hz` keyword can be used to ensure
+       that limits are in factors of decades in Hz, so that Bode plots with
+       `Hz=True` look better.
+
+    4. The frequency response data can be plotted by calling the
+       :func:`~control_bode_plot` function or using the `plot` method of
+       the :class:`~control.FrequencyResponseData` class.
 
     Examples
     --------
@@ -524,15 +475,63 @@ def frequency_response(sys, omega, squeeze=None):
         #>>> # s = 0.1i, i, 10i.
 
     """
-    return sys.frequency_response(omega, squeeze=squeeze)
+    from .frdata import FrequencyResponseData
+    from .freqplot import _determine_omega_vector
 
+    # Process keyword arguments
+    omega_num = config._get_param('freqplot', 'number_of_samples', omega_num)
+
+    # Convert the first argument to a list
+    syslist = sysdata if isinstance(sysdata, (list, tuple)) else [sysdata]
+
+    # Get the common set of frequencies to use
+    omega_syslist, omega_range_given = _determine_omega_vector(
+        syslist, omega, omega_limits, omega_num, Hz=Hz)
+
+    responses = []
+    for sys_ in syslist:
+        if isinstance(sys_, FrequencyResponseData) and sys_.ifunc is None and \
+           not omega_range_given:
+            omega_sys = sys_.omega              # use system properties
+        else:
+            omega_sys = omega_syslist.copy()    # use common omega vector
+
+            # Add the Nyquist frequency for discrete time systems
+            if sys_.isdtime(strict=True):
+                nyquistfrq = math.pi / sys_.dt
+                if not omega_range_given:
+                    # Limit up to the Nyquist frequency
+                    omega_sys = omega_sys[omega_sys < nyquistfrq]
+
+        # Compute the frequency response
+        responses.append(sys_.frequency_response(omega_sys, squeeze=squeeze))
+
+    if isinstance(sysdata, (list, tuple)):
+        from .freqplot import FrequencyResponseList
+        return FrequencyResponseList(responses)
+    else:
+        return responses[0]
 
 # Alternative name (legacy)
-freqresp = frequency_response
+def freqresp(sys, omega):
+    """Legacy version of frequency_response.
+
+    .. deprecated:: 0.9.0
+        This function will be removed in a future version of python-control.
+        Use `frequency_response` instead.
+
+    """
+    warn("freqresp() is deprecated; use frequency_response()", FutureWarning)
+    return frequency_response(sys, omega)
 
 
 def dcgain(sys):
-    """Return the zero-frequency (or DC) gain of the given system
+    """Return the zero-frequency (or DC) gain of the given system.
+
+    Parameters
+    ----------
+    sys : LTI
+        System for which the zero-frequency gain is computed.
 
     Returns
     -------
@@ -545,7 +544,7 @@ def dcgain(sys):
     --------
     >>> G = ct.tf([1], [1, 2])
     >>> ct.dcgain(G)                                            # doctest: +SKIP
-    0.5
+    np.float(0.5)
 
     """
     return sys.dcgain()
@@ -556,11 +555,11 @@ def bandwidth(sys, dbdrop=-3):
 
     Parameters
     ----------
-    sys: StateSpace or TransferFunction
-        Linear system
+    sys : StateSpace or TransferFunction
+        Linear system for which the bandwidth should be computed.
     dbdrop : float, optional
         By how much the gain drop in dB (default = -3) that defines the
-        bandwidth. Should be a negative scalar
+        bandwidth. Should be a negative scalar.
 
     Returns
     -------
