@@ -6,18 +6,21 @@ This is a rudimentary set of tests for frequency response functions,
 including bode plots.
 """
 
+import math
+import re
+
 import matplotlib.pyplot as plt
 import numpy as np
-from numpy.testing import assert_allclose
-import math
 import pytest
+from numpy.testing import assert_allclose
 
 import control as ctrl
+from control.freqplot import (bode_plot, nyquist_plot, nyquist_response,
+                              singular_values_plot, singular_values_response)
+from control.matlab import bode, rss, ss, tf
 from control.statesp import StateSpace
-from control.xferfcn import TransferFunction
-from control.matlab import ss, tf, bode, rss
-from control.freqplot import bode_plot, nyquist_plot, singular_values_plot
 from control.tests.conftest import slycotonly
+from control.xferfcn import TransferFunction
 
 pytestmark = pytest.mark.usefixtures("mplcleanup")
 
@@ -40,16 +43,26 @@ def ss_mimo():
     return StateSpace(A, B, C, D)
 
 
+@pytest.mark.filterwarnings("ignore:freqresp is deprecated")
+def test_freqresp_siso_legacy(ss_siso):
+    """Test SISO frequency response"""
+    omega = np.linspace(10e-2, 10e2, 1000)
+
+    # test frequency response
+    ctrl.frequency_response(ss_siso, omega)
+
+
 def test_freqresp_siso(ss_siso):
     """Test SISO frequency response"""
     omega = np.linspace(10e-2, 10e2, 1000)
 
     # test frequency response
-    ctrl.freqresp(ss_siso, omega)
+    ctrl.frequency_response(ss_siso, omega)
 
 
+@pytest.mark.filterwarnings(r"ignore:freqresp\(\) is deprecated")
 @slycotonly
-def test_freqresp_mimo(ss_mimo):
+def test_freqresp_mimo_legacy(ss_mimo):
     """Test MIMO frequency response calls"""
     omega = np.linspace(10e-2, 10e2, 1000)
     ctrl.freqresp(ss_mimo, omega)
@@ -57,6 +70,16 @@ def test_freqresp_mimo(ss_mimo):
     ctrl.freqresp(tf_mimo, omega)
 
 
+@slycotonly
+def test_freqresp_mimo(ss_mimo):
+    """Test MIMO frequency response calls"""
+    omega = np.linspace(10e-2, 10e2, 1000)
+    ctrl.frequency_response(ss_mimo, omega)
+    tf_mimo = tf(ss_mimo)
+    ctrl.frequency_response(tf_mimo, omega)
+
+
+@pytest.mark.usefixtures("legacy_plot_signature")
 def test_bode_basic(ss_siso):
     """Test bode plot call (Very basic)"""
     # TODO: proper test
@@ -77,21 +100,25 @@ def test_nyquist_basic(ss_siso):
     tf_siso = tf(ss_siso)
     nyquist_plot(ss_siso)
     nyquist_plot(tf_siso)
-    count, contour = nyquist_plot(
-        tf_siso, plot=False, return_contour=True, omega_num=20)
-    assert len(contour) == 20
+    response = nyquist_response(tf_siso, omega_num=20)
+    assert len(response.contour) == 20
 
-    with pytest.warns(UserWarning, match="encirclements was a non-integer"):
+    with pytest.warns() as record:
         count, contour = nyquist_plot(
             tf_siso, plot=False, omega_limits=(1, 100), return_contour=True)
     assert_allclose(contour[0], 1j)
     assert_allclose(contour[-1], 100j)
 
-    count, contour = nyquist_plot(
-        tf_siso, plot=False, omega=np.logspace(-1, 1, 10), return_contour=True)
-    assert len(contour) == 10
+    # Check known warnings happened as expected
+    assert len(record) == 2
+    assert re.search("encirclements was a non-integer", str(record[0].message))
+    assert re.search("return value .* deprecated", str(record[1].message))
+
+    response = nyquist_response(tf_siso, omega=np.logspace(-1, 1, 10))
+    assert len(response.contour) == 10
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 @pytest.mark.filterwarnings("ignore:.*non-positive left xlim:UserWarning")
 def test_superimpose():
     """Test superimpose multiple calls.
@@ -144,6 +171,7 @@ def test_superimpose():
     assert len(ax.get_lines()) == 2
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 def test_doubleint():
     """Test typcast bug with double int
 
@@ -157,6 +185,7 @@ def test_doubleint():
     bode(sys)
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 @pytest.mark.parametrize(
     "Hz, Wcp, Wcg",
     [pytest.param(False, 6.0782869, 10., id="omega"),
@@ -177,31 +206,32 @@ def test_bode_margin(dB, maginfty1, maginfty2, gminv,
     den = [1, 25, 100, 0]
     sys = ctrl.tf(num, den)
     plt.figure()
-    ctrl.bode_plot(sys, margins=True, dB=dB, deg=deg, Hz=Hz)
+    ctrl.bode_plot(sys, display_margins=True, dB=dB, deg=deg, Hz=Hz)
     fig = plt.gcf()
     allaxes = fig.get_axes()
 
+    # TODO: update with better tests for new margin plots
     mag_to_infinity = (np.array([Wcp, Wcp]),
                        np.array([maginfty1, maginfty2]))
-    assert_allclose(mag_to_infinity,
-                    allaxes[0].lines[2].get_data(),
+    assert_allclose(mag_to_infinity[0],
+                    allaxes[0].lines[2].get_data()[0],
                     rtol=1e-5)
 
     gm_to_infinty = (np.array([Wcg, Wcg]),
                      np.array([gminv, maginfty2]))
-    assert_allclose(gm_to_infinty,
-                    allaxes[0].lines[3].get_data(),
+    assert_allclose(gm_to_infinty[0],
+                    allaxes[0].lines[3].get_data()[0],
                     rtol=1e-5)
 
     one_to_gm = (np.array([Wcg, Wcg]),
                  np.array([maginfty1, gminv]))
-    assert_allclose(one_to_gm, allaxes[0].lines[4].get_data(),
+    assert_allclose(one_to_gm[0], allaxes[0].lines[4].get_data()[0],
                     rtol=1e-5)
 
     pm_to_infinity = (np.array([Wcp, Wcp]),
                       np.array([1e5, pm]))
-    assert_allclose(pm_to_infinity,
-                    allaxes[1].lines[2].get_data(),
+    assert_allclose(pm_to_infinity[0],
+                    allaxes[1].lines[2].get_data()[0],
                     rtol=1e-5)
 
     pm_to_phase = (np.array([Wcp, Wcp]),
@@ -211,7 +241,7 @@ def test_bode_margin(dB, maginfty1, maginfty2, gminv,
 
     phase_to_infinity = (np.array([Wcg, Wcg]),
                          np.array([0, p0]))
-    assert_allclose(phase_to_infinity, allaxes[1].lines[4].get_data(),
+    assert_allclose(phase_to_infinity[0], allaxes[1].lines[4].get_data()[0],
                     rtol=1e-5)
 
 
@@ -241,6 +271,7 @@ def dsystem_type(request, dsystem_dt):
     return dsystem_dt[systype]
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 @pytest.mark.parametrize("dsystem_dt", [0.1, True], indirect=True)
 @pytest.mark.parametrize("dsystem_type", ['sssiso', 'ssmimo', 'tf'],
                          indirect=True)
@@ -273,10 +304,12 @@ def test_discrete(dsystem_type):
 
     else:
         # Calling bode should generate a not implemented error
-        with pytest.raises(NotImplementedError):
-            bode((dsys,))
+        # with pytest.raises(NotImplementedError):
+        # TODO: check results
+        bode((dsys,))
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 def test_options(editsdefaults):
     """Test ability to set parameter values"""
     # Generate a Bode plot of a transfer function
@@ -309,6 +342,7 @@ def test_options(editsdefaults):
     assert numpoints1 != numpoints3
     assert numpoints3 == 13
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 @pytest.mark.parametrize(
     "TF, initial_phase, default_phase, expected_phase",
     [pytest.param(ctrl.tf([1], [1, 0]),
@@ -332,11 +366,11 @@ def test_options(editsdefaults):
      ])
 def test_initial_phase(TF, initial_phase, default_phase, expected_phase):
     # Check initial phase of standard transfer functions
-    mag, phase, omega = ctrl.bode(TF)
+    mag, phase, omega = ctrl.bode(TF, plot=True)
     assert(abs(phase[0] - default_phase) < 0.1)
 
     # Now reset the initial phase to +180 and see if things work
-    mag, phase, omega = ctrl.bode(TF, initial_phase=initial_phase)
+    mag, phase, omega = ctrl.bode(TF, initial_phase=initial_phase, plot=True)
     assert(abs(phase[0] - expected_phase) < 0.1)
 
     # Make sure everything works in rad/sec as well
@@ -344,10 +378,12 @@ def test_initial_phase(TF, initial_phase, default_phase, expected_phase):
         plt.xscale('linear')  # avoids xlim warning on next line
         plt.clf()  # clear previous figure (speeds things up)
         mag, phase, omega = ctrl.bode(
-            TF, initial_phase=initial_phase/180. * math.pi, deg=False)
+            TF, initial_phase=initial_phase/180. * math.pi,
+            deg=False, plot=True)
         assert(abs(phase[0] - expected_phase) < 0.1)
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 @pytest.mark.parametrize(
     "TF, wrap_phase, min_phase, max_phase",
     [pytest.param(ctrl.tf([1], [1, 0]),
@@ -370,11 +406,12 @@ def test_initial_phase(TF, initial_phase, default_phase, expected_phase):
                   -270, -3*math.pi/2, math.pi/2,    id="order5, -270"),
     ])
 def test_phase_wrap(TF, wrap_phase, min_phase, max_phase):
-    mag, phase, omega = ctrl.bode(TF, wrap_phase=wrap_phase)
+    mag, phase, omega = ctrl.bode(TF, wrap_phase=wrap_phase, plot=True)
     assert(min(phase) >= min_phase)
     assert(max(phase) <= max_phase)
 
 
+@pytest.mark.usefixtures("legacy_plot_signature")
 def test_phase_wrap_multiple_systems():
     sys_unstable = ctrl.zpk([],[1,1], gain=1)
 
@@ -398,14 +435,25 @@ def test_freqresp_warn_infinite():
     np.testing.assert_almost_equal(sys_finite(0, warn_infinite=True), 100)
 
     # Transfer function with infinite zero frequency gain
-    with pytest.warns(RuntimeWarning, match="divide by zero"):
+    with pytest.warns() as record:
         np.testing.assert_almost_equal(
             sys_infinite(0), complex(np.inf, np.nan))
-    with pytest.warns(RuntimeWarning, match="divide by zero"):
+    assert len(record) == 2     # generates two RuntimeWarnings
+    assert record[0].category is RuntimeWarning
+    assert re.search("divide by zero", str(record[0].message))
+    assert record[1].category is RuntimeWarning
+    assert re.search("invalid value", str(record[1].message))
+
+    with pytest.warns() as record:
         np.testing.assert_almost_equal(
             sys_infinite(0, warn_infinite=True), complex(np.inf, np.nan))
     np.testing.assert_almost_equal(
         sys_infinite(0, warn_infinite=False), complex(np.inf, np.nan))
+    assert len(record) == 2     # generates two RuntimeWarnings
+    assert record[0].category is RuntimeWarning
+    assert re.search("divide by zero", str(record[0].message))
+    assert record[1].category is RuntimeWarning
+    assert re.search("invalid value", str(record[1].message))
 
     # Switch to state space
     sys_finite = ctrl.tf2ss(sys_finite)
@@ -624,7 +672,8 @@ def tsystem(request, ss_mimo_ct, ss_miso_ct, ss_simo_ct, ss_siso_ct, ss_mimo_dt)
 def test_singular_values_plot(tsystem):
     sys = tsystem.sys
     for omega_ref, sigma_ref in zip(tsystem.omegas, tsystem.sigmas):
-        sigma, _ = singular_values_plot(sys, omega_ref, plot=False)
+        response = singular_values_response(sys, omega_ref)
+        sigma = np.real(response.fresp[:, 0, :])
         np.testing.assert_almost_equal(sigma, sigma_ref)
 
 
@@ -632,13 +681,13 @@ def test_singular_values_plot_mpl_base(ss_mimo_ct, ss_mimo_dt):
     sys_ct = ss_mimo_ct.sys
     sys_dt = ss_mimo_dt.sys
     plt.figure()
-    singular_values_plot(sys_ct, plot=True)
+    singular_values_plot(sys_ct)
     fig = plt.gcf()
     allaxes = fig.get_axes()
     assert(len(allaxes) == 1)
     assert(allaxes[0].get_label() == 'control-sigma')
     plt.figure()
-    singular_values_plot([sys_ct, sys_dt], plot=True, Hz=True, dB=True, grid=False)
+    singular_values_plot([sys_ct, sys_dt], Hz=True, dB=True, grid=False)
     fig = plt.gcf()
     allaxes = fig.get_axes()
     assert(len(allaxes) == 1)
@@ -648,10 +697,10 @@ def test_singular_values_plot_mpl_base(ss_mimo_ct, ss_mimo_dt):
 def test_singular_values_plot_mpl_superimpose_nyq(ss_mimo_ct, ss_mimo_dt):
     sys_ct = ss_mimo_ct.sys
     sys_dt = ss_mimo_dt.sys
-    omega_all = np.logspace(-3, 2, 1000)
+    omega_all = np.logspace(-3, int(math.log10(2 * math.pi/sys_dt.dt)), 1000)
     plt.figure()
-    singular_values_plot(sys_ct, omega_all, plot=True)
-    singular_values_plot(sys_dt, omega_all, plot=True)
+    singular_values_plot(sys_ct, omega_all)
+    singular_values_plot(sys_dt, omega_all)
     fig = plt.gcf()
     allaxes = fig.get_axes()
     assert(len(allaxes) == 1)
@@ -660,3 +709,25 @@ def test_singular_values_plot_mpl_superimpose_nyq(ss_mimo_ct, ss_mimo_dt):
     assert(len(nyquist_line[0]) == 2)
     assert(nyquist_line[0][0] == nyquist_line[0][1])
     assert(nyquist_line[0][0] == np.pi/sys_dt.dt)
+
+
+def test_freqresp_omega_limits():
+    sys = ctrl.rss(4, 1, 1)
+
+    # Generate a standard frequency response (no limits specified)
+    resp0 = ctrl.frequency_response(sys)
+
+    # Regenerate the response using omega_limits
+    resp1 = ctrl.frequency_response(
+        sys, omega_limits=[resp0.omega[0], resp0.omega[-1]])
+    np.testing.assert_equal(resp0.omega, resp1.omega)
+
+    # Regenerate the response using omega as a list of two elements
+    resp2 = ctrl.frequency_response(sys, [resp0.omega[0], resp0.omega[-1]])
+    np.testing.assert_equal(resp0.omega, resp2.omega)
+    assert resp2.omega.size > 100
+
+    # Make sure that generating response using array does the right thing
+    resp3 = ctrl.frequency_response(
+        sys, np.array([resp0.omega[0], resp0.omega[-1]]))
+    np.testing.assert_equal(resp3.omega, [resp0.omega[0], resp0.omega[-1]])
