@@ -8,11 +8,13 @@ from ipython to generate plots interactively.
 
 """
 
+import re
 import warnings
 
-import pytest
-import numpy as np
 import matplotlib.pyplot as plt
+import numpy as np
+import pytest
+
 import control as ct
 
 pytestmark = pytest.mark.usefixtures("mplcleanup")
@@ -40,7 +42,7 @@ def _Z(sys):
 def test_nyquist_basic():
     # Simple Nyquist plot
     sys = ct.rss(5, 1, 1)
-    N_sys = ct.nyquist_plot(sys)
+    N_sys = ct.nyquist_response(sys)
     assert _Z(sys) == N_sys + _P(sys)
 
     # Previously identified bug
@@ -62,17 +64,20 @@ def test_nyquist_basic():
     sys = ct.ss(A, B, C, D)
 
     # With a small indent_radius, all should be fine
-    N_sys = ct.nyquist_plot(sys, indent_radius=0.001)
+    N_sys = ct.nyquist_response(sys, indent_radius=0.001)
     assert _Z(sys) == N_sys + _P(sys)
 
     # With a larger indent_radius, we get a warning message + wrong answer
-    with pytest.warns(UserWarning, match="contour may miss closed loop pole"):
-        N_sys = ct.nyquist_plot(sys, indent_radius=0.2)
+    with pytest.warns() as rec:
+        N_sys = ct.nyquist_response(sys, indent_radius=0.2)
         assert _Z(sys) != N_sys + _P(sys)
+    assert len(rec) == 2
+    assert re.search("contour may miss closed loop pole", str(rec[0].message))
+    assert re.search("encirclements does not match", str(rec[1].message))
 
     # Unstable system
     sys = ct.tf([10], [1, 2, 2, 1])
-    N_sys = ct.nyquist_plot(sys)
+    N_sys = ct.nyquist_response(sys)
     assert _Z(sys) > 0
     assert _Z(sys) == N_sys + _P(sys)
 
@@ -80,14 +85,14 @@ def test_nyquist_basic():
     sys1 = ct.rss(3, 1, 1)
     sys2 = ct.rss(4, 1, 1)
     sys3 = ct.rss(5, 1, 1)
-    counts = ct.nyquist_plot([sys1, sys2, sys3])
+    counts = ct.nyquist_response([sys1, sys2, sys3])
     for N_sys, sys in zip(counts, [sys1, sys2, sys3]):
         assert _Z(sys) == N_sys + _P(sys)
 
     # Nyquist plot with poles at the origin, omega specified
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0])
     omega = np.linspace(0, 1e2, 100)
-    count, contour = ct.nyquist_plot(sys, omega, return_contour=True)
+    count, contour = ct.nyquist_response(sys, omega, return_contour=True)
     np.testing.assert_array_equal(
         contour[contour.real < 0], omega[contour.real < 0])
 
@@ -100,50 +105,53 @@ def test_nyquist_basic():
     # Make sure that we can turn off frequency modification
     #
     # Start with a case where indentation should occur
-    count, contour_indented = ct.nyquist_plot(
+    count, contour_indented = ct.nyquist_response(
         sys, np.linspace(1e-4, 1e2, 100), indent_radius=1e-2,
         return_contour=True)
     assert not all(contour_indented.real == 0)
-    with pytest.warns(UserWarning, match="encirclements does not match"):
-        count, contour = ct.nyquist_plot(
+
+    with pytest.warns() as record:
+        count, contour = ct.nyquist_response(
             sys, np.linspace(1e-4, 1e2, 100), indent_radius=1e-2,
             return_contour=True, indent_direction='none')
     np.testing.assert_almost_equal(contour, 1j*np.linspace(1e-4, 1e2, 100))
+    assert len(record) == 2
+    assert re.search("encirclements .* non-integer", str(record[0].message))
+    assert re.search("encirclements does not match", str(record[1].message))
 
     # Nyquist plot with poles at the origin, omega unspecified
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0])
-    count, contour = ct.nyquist_plot(sys, return_contour=True)
+    count, contour = ct.nyquist_response(sys, return_contour=True)
     assert _Z(sys) == count + _P(sys)
 
     # Nyquist plot with poles at the origin, return contour
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0])
-    count, contour = ct.nyquist_plot(sys, return_contour=True)
+    count, contour = ct.nyquist_response(sys, return_contour=True)
     assert _Z(sys) == count + _P(sys)
 
     # Nyquist plot with poles on imaginary axis, omega specified
     # (can miss encirclements due to the imaginary poles at +/- 1j)
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0, 1])
-    with pytest.warns(UserWarning, match="does not match") as records:
-        count = ct.nyquist_plot(sys, np.linspace(1e-3, 1e1, 1000))
-    if len(records) == 0:
-        assert _Z(sys) == count + _P(sys)
-
-    # Nyquist plot with poles on imaginary axis, omega specified, with contour
-    sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0, 1])
-    with pytest.warns(UserWarning, match="does not match") as records:
-        count, contour = ct.nyquist_plot(
-            sys, np.linspace(1e-3, 1e1, 1000), return_contour=True)
-    if len(records) == 0:
-        assert _Z(sys) == count + _P(sys)
+    with warnings.catch_warnings(record=True) as records:
+        count = ct.nyquist_response(sys, np.linspace(1e-3, 1e1, 1000))
+        if len(records) == 0:
+            # No warnings (it happens) => make sure count is correct
+            assert _Z(sys) == count + _P(sys)
+        elif len(records) == 1:
+            # Expected case: make sure warning is the right one
+            assert issubclass(records[0].category, UserWarning)
+            assert "encirclements does not match" in str(records[0].message)
+        else:
+            pytest.fail("multiple warnings in nyquist_response (?)")
 
     # Nyquist plot with poles on imaginary axis, return contour
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0, 1])
-    count, contour = ct.nyquist_plot(sys, return_contour=True)
+    count, contour = ct.nyquist_response(sys, return_contour=True)
     assert _Z(sys) == count + _P(sys)
 
     # Nyquist plot with poles at the origin and on imaginary axis
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0, 1]) * ct.tf([1], [1, 0])
-    count, contour = ct.nyquist_plot(sys, return_contour=True)
+    count, contour = ct.nyquist_response(sys, return_contour=True)
     assert _Z(sys) == count + _P(sys)
 
 
@@ -153,36 +161,43 @@ def test_nyquist_fbs_examples():
 
     """Run through various examples from FBS2e to compare plots"""
     plt.figure()
-    plt.title("Figure 10.4: L(s) = 1.4 e^{-s}/(s+1)^2")
     sys = ct.tf([1.4], [1, 2, 1]) * ct.tf(*ct.pade(1, 4))
-    count = ct.nyquist_plot(sys)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Figure 10.4: L(s) = 1.4 e^{-s}/(s+1)^2")
+    assert _Z(sys) == response.count + _P(sys)
 
     plt.figure()
-    plt.title("Figure 10.4: L(s) = 1/(s + a)^2 with a = 0.6")
     sys = 1/(s + 0.6)**3
-    count = ct.nyquist_plot(sys)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Figure 10.4: L(s) = 1/(s + a)^2 with a = 0.6")
+    assert _Z(sys) == response.count + _P(sys)
 
     plt.figure()
-    plt.title("Figure 10.6: L(s) = 1/(s (s+1)^2) - pole at the origin")
     sys = 1/(s * (s+1)**2)
-    count = ct.nyquist_plot(sys)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title(
+        "Figure 10.6: L(s) = 1/(s (s+1)^2) - pole at the origin")
+    assert _Z(sys) == response.count + _P(sys)
 
     plt.figure()
-    plt.title("Figure 10.10: L(s) = 3 (s+6)^2 / (s (s+1)^2)")
     sys = 3 * (s+6)**2 / (s * (s+1)**2)
-    count = ct.nyquist_plot(sys)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Figure 10.10: L(s) = 3 (s+6)^2 / (s (s+1)^2)")
+    assert _Z(sys) == response.count + _P(sys)
 
     plt.figure()
-    plt.title("Figure 10.10: L(s) = 3 (s+6)^2 / (s (s+1)^2) [zoom]")
     with pytest.warns(UserWarning, match="encirclements does not match"):
-        count = ct.nyquist_plot(sys, omega_limits=[1.5, 1e3])
+        response = ct.nyquist_response(sys, omega_limits=[1.5, 1e3])
+        cplt = response.plot()
+        cplt.set_plot_title(
+            "Figure 10.10: L(s) = 3 (s+6)^2 / (s (s+1)^2) [zoom]")
         # Frequency limits for zoom give incorrect encirclement count
-        # assert _Z(sys) == count + _P(sys)
-        assert count == -1
+        # assert _Z(sys) == response.count + _P(sys)
+        assert response.count == -1
 
 
 @pytest.mark.parametrize("arrows", [
@@ -194,9 +209,26 @@ def test_nyquist_fbs_examples():
 def test_nyquist_arrows(arrows):
     sys = ct.tf([1.4], [1, 2, 1]) * ct.tf(*ct.pade(1, 4))
     plt.figure();
-    plt.title("L(s) = 1.4 e^{-s}/(s+1)^2 / arrows = %s" % arrows)
-    count = ct.nyquist_plot(sys, arrows=arrows)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot(arrows=arrows)
+    cplt.set_plot_title("L(s) = 1.4 e^{-s}/(s+1)^2 / arrows = %s" % arrows)
+    assert _Z(sys) == response.count + _P(sys)
+
+
+def test_sensitivity_circles():
+    A = np.array([
+        [-3.56355873, -1.22980795, -1.5626527 , -0.4626829],
+        [-8.52361371, -3.60331459, -3.71574266, -0.43839201],
+        [-2.50458726, -0.72361335, -1.77795489, -0.4038419],
+        [-0.281183  ,  0.23391825,  0.19096003, -0.9771515]])
+    B = np.array([[-0.], [-1.42827213], [ 0.76806551], [-1.07987454]])
+    C = np.array([[-0.,  0.35557249,  0.35941791, -0.]])
+    D = np.array([[0]])
+    sys1 = ct.ss(A, B, C, D)
+    sys2 = ct.ss(A, B, C, D, dt=0.1)
+    plt.figure()
+    ct.nyquist_plot(sys1, unit_circle=True, mt_circles=[0.9,1,1.1,1.2], ms_circles=[0.9,1,1.1,1.2])
+    ct.nyquist_plot(sys2, unit_circle=True, mt_circles=[0.9,1,1.1,1.2], ms_circles=[0.9,1,1.1,1.2])
 
 
 def test_nyquist_encirclements():
@@ -205,34 +237,40 @@ def test_nyquist_encirclements():
     sys = (0.02 * s**3 - 0.1 * s) / (s**4 + s**3 + s**2 + 0.25 * s + 0.04)
 
     plt.figure();
-    count = ct.nyquist_plot(sys)
-    plt.title("Stable system; encirclements = %d" % count)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Stable system; encirclements = %d" % response.count)
+    assert _Z(sys) == response.count + _P(sys)
 
     plt.figure();
-    count = ct.nyquist_plot(sys * 3)
-    plt.title("Unstable system; encirclements = %d" % count)
-    assert _Z(sys * 3) == count + _P(sys * 3)
+    response = ct.nyquist_response(sys * 3)
+    cplt = response.plot()
+    cplt.set_plot_title("Unstable system; encirclements = %d" %response.count)
+    assert _Z(sys * 3) == response.count + _P(sys * 3)
 
     # System with pole at the origin
     sys = ct.tf([3], [1, 2, 2, 1, 0])
 
     plt.figure();
-    count = ct.nyquist_plot(sys)
-    plt.title("Pole at the origin; encirclements = %d" % count)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title(
+        "Pole at the origin; encirclements = %d" %response.count)
+    assert _Z(sys) == response.count + _P(sys)
 
     # Non-integer number of encirclements
     plt.figure();
     sys = 1 / (s**2 + s + 1)
     with pytest.warns(UserWarning, match="encirclements was a non-integer"):
-        count = ct.nyquist_plot(sys, omega_limits=[0.5, 1e3])
+        response = ct.nyquist_response(sys, omega_limits=[0.5, 1e3])
     with warnings.catch_warnings():
         warnings.simplefilter("error")
         # strip out matrix warnings
-        count = ct.nyquist_plot(
+        response = ct.nyquist_response(
             sys, omega_limits=[0.5, 1e3], encirclement_threshold=0.2)
-    plt.title("Non-integer number of encirclements [%g]" % count)
+        cplt = response.plot()
+    cplt.set_plot_title(
+        "Non-integer number of encirclements [%g]" %response.count)
 
 
 @pytest.fixture
@@ -245,41 +283,61 @@ def indentsys():
 
 def test_nyquist_indent_default(indentsys):
     plt.figure();
-    count = ct.nyquist_plot(indentsys)
-    plt.title("Pole at origin; indent_radius=default")
-    assert _Z(indentsys) == count + _P(indentsys)
+    response = ct.nyquist_response(indentsys)
+    cplt = response.plot()
+    cplt.set_plot_title("Pole at origin; indent_radius=default")
+    assert _Z(indentsys) == response.count + _P(indentsys)
 
 
 def test_nyquist_indent_dont(indentsys):
     # first value of default omega vector was 0.1, replaced by 0. for contour
     # indent_radius is larger than 0.1 -> no extra quater circle around origin
-    with pytest.warns(UserWarning, match="encirclements does not match"):
-        count, contour = ct.nyquist_plot(
+    with pytest.warns() as record:
+        count, contour = ct.nyquist_response(
             indentsys, omega=[0, 0.2, 0.3, 0.4], indent_radius=.1007,
-            plot=False, return_contour=True)
+            return_contour=True)
     np.testing.assert_allclose(contour[0], .1007+0.j)
     # second value of omega_vector is larger than indent_radius: not indented
     assert np.all(contour.real[2:] == 0.)
 
+    # Make sure warnings are as expected
+    assert len(record) == 2
+    assert re.search("encirclements .* non-integer", str(record[0].message))
+    assert re.search("encirclements does not match", str(record[1].message))
+
 
 def test_nyquist_indent_do(indentsys):
     plt.figure();
-    count, contour = ct.nyquist_plot(
+    response = ct.nyquist_response(
         indentsys, indent_radius=0.01, return_contour=True)
-    plt.title("Pole at origin; indent_radius=0.01; encirclements = %d" % count)
+    count, contour = response
+    cplt = response.plot()
+    cplt.set_plot_title(
+        "Pole at origin; indent_radius=0.01; encirclements = %d" % count)
     assert _Z(indentsys) == count + _P(indentsys)
     # indent radius is smaller than the start of the default omega vector
     # check that a quarter circle around the pole at origin has been added.
     np.testing.assert_allclose(contour[:50].real**2 + contour[:50].imag**2,
                                0.01**2)
 
+    # Make sure that the command also works if called directly as _plot()
+    plt.figure()
+    with pytest.warns(FutureWarning, match=".* use nyquist_response()"):
+        count, contour = ct.nyquist_plot(
+            indentsys, indent_radius=0.01, return_contour=True)
+    assert _Z(indentsys) == count + _P(indentsys)
+    np.testing.assert_allclose(
+        contour[:50].real**2 + contour[:50].imag**2, 0.01**2)
+
 
 def test_nyquist_indent_left(indentsys):
     plt.figure();
-    count = ct.nyquist_plot(indentsys, indent_direction='left')
-    plt.title(
-        "Pole at origin; indent_direction='left'; encirclements = %d" % count)
-    assert _Z(indentsys) == count + _P(indentsys, indent='left')
+    response = ct.nyquist_response(indentsys, indent_direction='left')
+    cplt = response.plot()
+    cplt.set_plot_title(
+        "Pole at origin; indent_direction='left'; encirclements = %d" %
+        response.count)
+    assert _Z(indentsys) == response.count + _P(indentsys, indent='left')
 
 
 def test_nyquist_indent_im():
@@ -288,25 +346,30 @@ def test_nyquist_indent_im():
 
     # Imaginary poles with standard indentation
     plt.figure();
-    count = ct.nyquist_plot(sys)
-    plt.title("Imaginary poles; encirclements = %d" % count)
-    assert _Z(sys) == count + _P(sys)
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Imaginary poles; encirclements = %d" % response.count)
+    assert _Z(sys) == response.count + _P(sys)
 
     # Imaginary poles with indentation to the left
     plt.figure();
-    count = ct.nyquist_plot(sys, indent_direction='left', label_freq=300)
-    plt.title(
-        "Imaginary poles; indent_direction='left'; encirclements = %d" % count)
-    assert _Z(sys) == count + _P(sys, indent='left')
+    response = ct.nyquist_response(sys, indent_direction='left')
+    cplt = response.plot(label_freq=300)
+    cplt.set_plot_title(
+        "Imaginary poles; indent_direction='left'; encirclements = %d" %
+        response.count)
+    assert _Z(sys) == response.count + _P(sys, indent='left')
 
     # Imaginary poles with no indentation
     plt.figure();
     with pytest.warns(UserWarning, match="encirclements does not match"):
-        count = ct.nyquist_plot(
+        response = ct.nyquist_response(
             sys, np.linspace(0, 1e3, 1000), indent_direction='none')
-    plt.title(
-        "Imaginary poles; indent_direction='none'; encirclements = %d" % count)
-    assert _Z(sys) == count + _P(sys)
+        cplt = response.plot()
+    cplt.set_plot_title(
+        "Imaginary poles; indent_direction='none'; encirclements = %d" %
+        response.count)
+    assert _Z(sys) == response.count + _P(sys)
 
 
 def test_nyquist_exceptions():
@@ -317,9 +380,9 @@ def test_nyquist_exceptions():
             match="only supports SISO"):
         ct.nyquist_plot(sys)
 
-    # Legacy keywords for arrow size
+    # Legacy keywords for arrow size (no longer supported)
     sys = ct.rss(2, 1, 1)
-    with pytest.warns(FutureWarning, match="use `arrow_size` instead"):
+    with pytest.raises(AttributeError):
         ct.nyquist_plot(sys, arrow_width=8, arrow_length=6)
 
     # Unknown arrow keyword
@@ -332,18 +395,26 @@ def test_nyquist_exceptions():
         ct.nyquist_plot(sys, indent_direction='up')
 
     # Discrete time system sampled above Nyquist frequency
-    sys = ct.drss(2, 1, 1)
-    sys.dt = 0.01
-    with pytest.warns(UserWarning, match="above Nyquist"):
+    sys = ct.ss([[-0.5, 0], [1, 0.5]], [[0], [1]], [[1, 0]], 0, 0.1)
+    with pytest.warns(UserWarning, match="evaluation above Nyquist"):
         ct.nyquist_plot(sys, np.logspace(-2, 3))
 
 
 def test_linestyle_checks():
-    sys = ct.rss(2, 1, 1)
+    sys = ct.tf([100], [1, 1, 1])
 
-    # Things that should work
-    ct.nyquist_plot(sys, primary_style=['-', '-'], mirror_style=['-', '-'])
-    ct.nyquist_plot(sys, mirror_style=None)
+    # Set the line styles
+    cplt = ct.nyquist_plot(
+        sys, primary_style=[':', ':'], mirror_style=[':', ':'])
+    assert all([line.get_linestyle() == ':' for line in cplt.lines[0]])
+
+    # Set the line colors
+    cplt = ct.nyquist_plot(sys, color='g')
+    assert all([line.get_color() == 'g' for line in cplt.lines[0]])
+
+    # Turn off the mirror image
+    cplt = ct.nyquist_plot(sys, mirror_style=False)
+    assert cplt.lines[0][2:] == [None, None]
 
     with pytest.raises(ValueError, match="invalid 'primary_style'"):
         ct.nyquist_plot(sys, primary_style=False)
@@ -365,26 +436,85 @@ def test_nyquist_legacy():
     sys = (0.02 * s**3 - 0.1 * s) / (s**4 + s**3 + s**2 + 0.25 * s + 0.04)
 
     with pytest.warns(UserWarning, match="indented contour may miss"):
-        count = ct.nyquist_plot(sys)
+        response = ct.nyquist_plot(sys)
 
 def test_discrete_nyquist():
+    # TODO: add tests to make sure plots make sense
+
     # Make sure we can handle discrete time systems with negative poles
     sys = ct.tf(1, [1, -0.1], dt=1) * ct.tf(1, [1, 0.1], dt=1)
-    ct.nyquist_plot(sys, plot=False)
+    ct.nyquist_response(sys)
 
     # system with a pole at the origin
     sys = ct.zpk([1,], [.3, 0], 1, dt=True)
-    ct.nyquist_plot(sys, plot=False)
+    ct.nyquist_response(sys)
     sys = ct.zpk([1,], [0], 1, dt=True)
-    ct.nyquist_plot(sys, plot=False)
+    ct.nyquist_response(sys)
 
     # only a pole at the origin
     sys = ct.zpk([], [0], 2, dt=True)
-    ct.nyquist_plot(sys, plot=False)
+    ct.nyquist_response(sys)
 
     # pole at zero (pure delay)
     sys = ct.zpk([], [1], 1, dt=True)
-    ct.nyquist_plot(sys, plot=False)
+    ct.nyquist_response(sys)
+
+
+def test_freqresp_omega_limits():
+    sys = ct.rss(4, 1, 1)
+
+    # Generate a standard frequency response (no limits specified)
+    resp0 = ct.nyquist_response(sys)
+    assert resp0.contour.size > 2
+
+    # Regenerate the response using omega_limits
+    resp1 = ct.nyquist_response(
+        sys, omega_limits=[resp0.contour[1].imag, resp0.contour[-1].imag])
+    assert resp1.contour.size > 2
+    assert np.isclose(resp1.contour[0], resp0.contour[1])
+    assert np.isclose(resp1.contour[-1], resp0.contour[-1])
+
+    # Regenerate the response using omega as a list of two elements
+    resp2 = ct.nyquist_response(
+        sys, [resp0.contour[1].imag, resp0.contour[-1].imag])
+    np.testing.assert_equal(resp1.contour, resp2.contour)
+
+    # Make sure that generating response using array does the right thing
+    resp3 = ct.nyquist_response(
+        sys, np.array([resp0.contour[1].imag, resp0.contour[-1].imag]))
+    np.testing.assert_equal(
+        resp3.contour,
+        np.array([resp0.contour[1], resp0.contour[-1]]))
+
+
+def test_nyquist_frd():
+    sys = ct.rss(4, 1, 1)
+    sys1 = ct.frd(sys, np.logspace(-1, 1, 10), name='sys1')
+    sys2 = ct.frd(sys, np.logspace(-2, 2, 10), name='sys2')
+    sys3 = ct.frd(sys, np.logspace(-2, 2, 10), smooth=True, name='sys3')
+
+    # Turn off warnings about number of encirclements
+    warnings.filterwarnings(
+        'ignore', message="number of encirclements was a non-integer value",
+        category=UserWarning)
+
+    # OK to specify frequency with FRD sys if frequencies match
+    nyqresp = ct.nyquist_response(sys1, np.logspace(-1, 1, 10))
+    np.testing.assert_allclose(nyqresp.contour, np.logspace(-1, 1, 10) * 1j)
+
+    # If a fixed FRD omega is used, generate an error on mismatch
+    with pytest.raises(ValueError, match="not all frequencies .* in .* list"):
+        nyqresp = ct.nyquist_response(sys2, np.logspace(-1, 1, 10))
+
+    # OK to specify frequency with FRD sys if interpolating FRD is used
+    nyqresp = ct.nyquist_response(sys3, np.logspace(-1, 1, 12))
+    np.testing.assert_allclose(nyqresp.contour, np.logspace(-1, 1, 12) * 1j)
+
+    # Computing Nyquist response w/ different frequencies OK if given as a list
+    nyqresp = ct.nyquist_response([sys1, sys2])
+    cplt = nyqresp.plot()
+
+    warnings.resetwarnings()
 
 
 if __name__ == "__main__":
@@ -410,6 +540,9 @@ if __name__ == "__main__":
     test_nyquist_arrows(3)
     test_nyquist_arrows([0.1, 0.5, 0.9])
 
+    print("Test sensitivity circles")
+    test_sensitivity_circles()
+
     print("Stability checks")
     test_nyquist_encirclements()
 
@@ -430,17 +563,26 @@ if __name__ == "__main__":
     print("Unusual Nyquist plot")
     sys = ct.tf([1], [1, 3, 2]) * ct.tf([1], [1, 0, 1])
     plt.figure()
-    plt.title("Poles: %s" %
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Poles: %s" %
               np.array2string(sys.poles(), precision=2, separator=','))
-    count = ct.nyquist_plot(sys)
-    assert _Z(sys) == count + _P(sys)
+    assert _Z(sys) == response.count + _P(sys)
 
     print("Discrete time systems")
     sys = ct.c2d(sys, 0.01)
     plt.figure()
-    plt.title("Discrete-time; poles: %s" %
+    response = ct.nyquist_response(sys)
+    cplt = response.plot()
+    cplt.set_plot_title("Discrete-time; poles: %s" %
               np.array2string(sys.poles(), precision=2, separator=','))
-    count = ct.nyquist_plot(sys)
-
-
 
+    print("Frequency response data (FRD) systems")
+    sys = ct.tf(
+        (0.02 * s**3 - 0.1 * s) / (s**4 + s**3 + s**2 + 0.25 * s + 0.04),
+        name='tf')
+    sys1 = ct.frd(sys, np.logspace(-1, 1, 15), name='frd1')
+    sys2 = ct.frd(sys, np.logspace(-2, 2, 20), name='frd2')
+    plt.figure()
+    cplt = ct.nyquist_plot([sys, sys1, sys2])
+    cplt.set_plot_title("Mixed FRD, tf data")
