@@ -10,21 +10,29 @@ import matplotlib.pyplot as plt
 
 plt.close('all')
 
-#controlable canonical realization computed in matlab for the transfer function:
-# num = [1 11 45 32], den = [1 15 60 200 60]
-A = np.matrix('-15., -7.5, -6.25, -1.875; \
-8., 0., 0., 0.; \
-0., 4., 0., 0.; \
-0., 0., 1., 0.')
-B = np.matrix('2.; 0.; 0.; 0.')
-C = np.matrix('0.5, 0.6875, 0.7031, 0.5')
-D = np.matrix('0.')
+# controllable canonical realization computed in MATLAB for the
+# transfer function: num = [1 11 45 32], den = [1 15 60 200 60]
+A = np.array([
+    [-15., -7.5, -6.25, -1.875],
+    [8., 0., 0., 0.],
+    [0., 4., 0., 0.],
+    [0., 0., 1., 0.]
+])
+B = np.array([
+    [2.],
+    [0.],
+    [0.],
+    [0.]
+])
+C = np.array([[0.5, 0.6875, 0.7031, 0.5]])
+D = np.array([[0.]])
 
 # The full system
-fsys = StateSpace(A,B,C,D)
+fsys = StateSpace(A, B, C, D)
+
 # The reduced system, truncating the order by 1
-ord = 3
-rsys = msimp.balred(fsys,ord, method = 'truncate')
+n = 3
+rsys = msimp.balred(fsys, n, method='truncate')
 
 # Comparison of the step responses of the full and reduced systems
 plt.figure(1)
@@ -35,15 +43,13 @@ plt.plot(tr.T, yr.T)
 
 # Repeat balanced reduction, now with 100-dimensional random state space
 sysrand = mt.rss(100, 1, 1)
-rsysrand = msimp.balred(sysrand,10,method ='truncate')
+rsysrand = msimp.balred(sysrand, 10, method='truncate')
 
 # Comparison of the impulse responses of the full and reduced random systems
 plt.figure(2)
 yrand, trand = mt.impulse(sysrand)
 yrandr, trandr = mt.impulse(rsysrand)
-plt.plot(trand.T, yrand.T, trandr.T, yrandr.T) 
-
+plt.plot(trand.T, yrand.T, trandr.T, yrandr.T)
 
 if 'PYCONTROL_TEST_EXAMPLES' not in os.environ:
     plt.show()
-
